/* eslint-disable no-restricted-imports */
import type { Strapi } from "@strapi/strapi";
import type { z } from "zod";

import type { requestBodySchema } from "../controllers/schema";
import type { Roles, UserDetails } from "../utils";
import { BadRequestError } from "../utils/error";

enum STAGES {
	approved = "Approved",
	rejected = "Rejected",
	hold = "Hold",
	in_progress = "In progress",
	ready_for_review = "Ready for review"
}
interface RelatedObj {
	__type: string;
	id: number;
}

export type DataObject = {
	stage: string;
	related: RelatedObj;
	created_by_id: number;
	approver?: number;
	assignee?: number;
};

type UserData = {
	id: number;
	firstname: string;
	lastname?: string;
	username?: string;
	roles: Roles[];
};

type AssigneeApprover = {
	assignee: number;
	approver: number;
};

type UpdateData = {
	id: number;
	data: Partial<CreateData> & Partial<AssigneeApprover>;
	user: UserData;
	updated_by_id: string;
};
type CreateData = z.infer<typeof requestBodySchema> & {
	id?: number;
};

export default ({ strapi }: { strapi: Strapi }) => ({
	async allStages(): Promise<STAGES[]> {
		return Object.values(STAGES);
	},

	async users() {
		const eligibleUsers = (await strapi.entityService?.findMany("admin::user", {
			fields: ["id", "firstname", "lastname", "username", "email"],
			filters: { blocked: false, isActive: true }
		})) as UserDetails[];

		const updatedArray = eligibleUsers.map((item) => ({
			...item,
			name: `${item.firstname} ${item?.lastname}`
		}));
		return updatedArray;
	},

	async roles() {
		const eligibleRoles = await strapi.entityService?.findMany("admin::role", {
			fields: ["id", "name", "code"]
		});

		return eligibleRoles;
	},

	async updateEntry(params: UpdateData) {
		const { id, data, user } = params;
		if (data.taskId) {
			data.id = data.taskId;
			delete data.taskId;
		}
		return await strapi.entityService?.update("plugin::review-workflow.workflow", id, {
			data: { ...data, updated_by_id: user.id }
		});
	},

	async deleteEntry(id) {
		return strapi.entityService?.delete("plugin::review-workflow.workflow", id);
	},
	async create(input: CreateData, user: UserData): Promise<DataObject | void> {
		const data = mapObject(input, user);

		const stage = `${data.stage}`.trim().toLowerCase();
		if (stage === "approved") {
			throw new BadRequestError("You can't transition to 'Approved' immediately");
		}
		const response = await strapi.entityService?.create("plugin::review-workflow.workflow", {
			data
		});
		return response as unknown as DataObject;
	}
});

const mapObject = (obj: CreateData, user: UserData): DataObject => {
	return {
		stage: obj.stage,
		related: {
			id: obj.taskId,
			__type: obj.type
		},
		created_by_id: user.id
	};
};
